@@ -2,7 +2,7 @@ package Text::Template::Simple;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.70';
+$VERSION = '0.80';
 
 use File::Spec;
 use Text::Template::Simple::Constants qw(:all);
@@ -48,8 +48,7 @@ my %DEFAULT = ( # default object attributes
    verbose_errors   =>  0,    # bool
    pre_chomp        => CHOMP_NONE,
    post_chomp       => CHOMP_NONE,
-   # TODO: Consider removing this
-   resume           =>  0,    # resume on error?
+   taint_mode       => TAINT_CHECK_NORMAL,
 );
 
 my @EXPORT_OK = qw( tts );
@@ -91,7 +90,7 @@ sub new {
    my($fid, $fval);
    INITIALIZE: foreach my $field ( keys %DEFAULT ) {
       $fid = uc $field;
-      next INITIALIZE if ! $class->can($fid);
+      next INITIALIZE if ! $class->can( $fid );
       $fid  = $class->$fid();
       $fval = delete $param{$field};
       $self->[$fid] = defined $fval ? $fval : $DEFAULT{$field};
@@ -144,7 +143,7 @@ sub _init {
    $self->[FAKER_HASH]     = $self->_output_buffer_var('hash');
    $self->[FAKER_SELF]     = $self->_output_buffer_var('self');
    $self->[INSIDE_INCLUDE] = -1; # must be -1 not 0
-   $self->[NEEDS_OBJECT]   =  0; # does the template need $self ?
+   $self->[NEEDS_OBJECT]   =  0; # the template needs $self ?
    $self->[DEEP_RECURSION] =  0; # recursion detector
 
    fatal('tts.main.init.thandler')
@@ -155,7 +154,8 @@ sub _init {
 
    $self->[IO_OBJECT] = $self->connector('IO')->new(
                            $self->[IOLAYER],
-                           $self->[INCLUDE_PATHS]
+                           $self->[INCLUDE_PATHS],
+                           $self->[TAINT_MODE],
                         );
 
    if ( $self->[CACHE_DIR] ) {
@@ -216,12 +216,18 @@ sub _tidy {
    return $buf;
 }
 
+sub _needs_object {
+   my $self = shift;
+   $self->[NEEDS_OBJECT]++;
+   return $self;
+}
+
 sub DESTROY {
    my $self = shift || return;
-   LOG( DESTROY => ref $self ) if DEBUG();
    undef $self->[CACHE_OBJECT];
    undef $self->[IO_OBJECT];
    @{ $self } = ();
+   LOG( DESTROY => ref $self ) if DEBUG();
    return;
 }
 
@@ -253,14 +259,14 @@ Where C<hello.tts> has this content:
 
 =head1 DESCRIPTION
 
-This document describes version C<0.70> of C<Text::Template::Simple>
-released on C<26 April 2009>.
+This document describes version C<0.80> of C<Text::Template::Simple>
+released on C<8 August 2009>.
 
 This is a simple template module. There is no extra template/mini 
-language. Instead, it uses Perl as a template language. Templates
-can be cached on disk or inside the memory via internal cache 
+language. Instead, it uses Perl as the template language. Templates
+can be cached on disk or inside the memory via the internal cache 
 manager. It is also possible to use static/dynamic includes,
-pass parameters to includes and allpt filters on them.
+pass parameters to includes and apply filters on them.
 Also see L<Text::Template::Simple::API> for the full API definiton.
 
 =head1 SYNTAX
@@ -271,31 +277,31 @@ Template syntax is very simple. There are few kinds of delimiters:
 
 =item *
 
-Code Blocks: C<< <% %> >>
+C<< <% %>  >> Code Blocks
 
 =item *
 
-Self-printing Blocks: C<< <%= %> >>
+C<< <%= %> >> Self-printing Blocks
 
 =item *
 
-Escaped Delimiters: C<< <%! %> >>
+C<< <%! %> >> Escaped Delimiters
 
 =item *
 
-Static Include Directives: C<< <%+ %> >>
+C<< <%+ %> >> Static Include Directives
 
 =item *
 
-Dynamic include directives C<< <%* %> >>
+C<< <%* %> >> Dynamic include directives
 
 =item *
 
-Comment Directives: C<< <%# %> >>
+C<< <%# %> >> Comment Directives
 
 =item *
 
-Blocks with commands: C<< <%| %> >>.
+C<< <%| %> >> Blocks with commands
 
 =back
 
@@ -354,7 +360,7 @@ parsing for the included files, use the dynamic includes:
    <%* my_other.html %>
    <%* my_other.txt  %>
 
-Interpolation is also supported with both kind of includes, so the following
+Interpolation is also supported with both kinds of includes, so the following
 is valid code:
 
    <%+ "/path/to/" . $txt    %>
@@ -500,13 +506,19 @@ PARAM
 
 FILTER
 
+=item *
+
+SHARE
+
 =back
 
    <%+ /path/to/static.tts  | FILTER: MyFilter %>
    <%* /path/to/dynamic.tts | FILTER: MyFilter | PARAM: test => 123 %>
 
-C<FILTER:> defines the list of filters to apply to the output of the include.
 C<PARAM:> defines the parameter list to pass to the included file.
+C<FILTER:> defines the list of filters to apply to the output of the include.
+C<SHARE:> used to list the variables to share with the included template when
+the monolith option is disabled.
 
 =head3 INCLUDE FILTERS
 
@@ -535,6 +547,38 @@ prefix:
 Just pass the parameters as described above and fetch them via C<@_> inside
 the included file.
 
+=head3 SHARED VARIABLES
+
+C<Text::Template::Simple> compiles every template individually with separate
+scopes. A variable defined in the master template is not accessible from a
+dynamic include. The exception to this rule is the C<monolith> option to C<new>.
+If it is enabled; the master template and any includes it has will be compiled
+into a single document, thus making every variable defined at the top available
+to the includes below. But this method has several drawbacks, it disables cache
+check for the sub files (includes) --you'll need to edit the master template
+to force a cache reload-- and it can not be used with interpolated includes.
+If you use an interpolated include with monolith enabled, you'll get an error.
+
+If you don't use C<monolith> (disabled by default), then you'll need to share
+the variables somehow to don't repeat yourself. Variable sharing is demonstrated
+in the below template:
+
+   <%
+      my $foo = 42;
+      my $bar = 23;
+   %>
+   <%* dyna.inc | SHARE: $foo, $bar %>
+
+And then you can access C<$foo> and C<$bar> inside C<dyna.inc>. There is one
+drawback by shared variables: only SCALARs can be shared. You can not share
+anything else. If you want to share an array, use an array reference instead:
+
+   <%
+      my @foo = (1..10);
+      my $fooref = \@foo;
+   %>
+   <%* dyna.inc | SHARE: $fooref %>
+
 =head2 BLOCKS
 
 A block consists of a header part and the content.
@@ -636,16 +680,16 @@ However, be aware that the monolithic version is B<not supported>.
 
 =head1 AUTHOR
 
-Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
+Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004 - 2009 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.8.8 or, 
+it under the same terms as Perl itself, either Perl version 5.10.0 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
