@@ -4,7 +4,7 @@ use vars qw($VERSION);
 use Text::Template::Simple::Util qw(:all);
 use Text::Template::Simple::Constants qw(:all);
 
-$VERSION = '0.70';
+$VERSION = '0.80';
 
 sub _compiler { shift->[SAFE] ? COMPILER_SAFE : COMPILER }
 
@@ -39,23 +39,26 @@ sub _compile {
    }
 
    if ( $opt->{chkmt} ) {
-      if ( $self->[TYPE] eq 'FILE' ) {
-         $opt->{chkmt} = (stat $tmpx)[STAT_MTIME];
-      }
-      else {
-         LOG( DISABLE_MT => "Disabling chkmt. Template is not a file" )
-            if DEBUG();
-         $opt->{chkmt} = 0;
-      }
+      $opt->{chkmt} = $self->[TYPE] eq 'FILE' ? (stat $tmpx)[STAT_MTIME]
+                    : do {
+                        DEBUG && LOG(DISABLE_MT =>
+                                     "Disabling chkmt. Template is not a file");
+                        0;
+                     }
    }
 
-   LOG( COMPILE => $opt->{id} ) if defined $opt->{id} && DEBUG();
+   LOG( COMPILE => $opt->{id} ) if DEBUG && defined $opt->{id};
 
    my($CODE, $ok);
    my $cache_id = '';
 
    my $as_is = $opt->{_sub_inc} && $opt->{_sub_inc} == T_STATIC;
 
+   # first element is the shared names. if it's not defined, then there
+   # are no shared variables from top level
+   delete $opt->{_share}
+      if isaref($opt->{_share}) && ! defined $opt->{_share}[0];
+
    if ( $self->[CACHE] ) {
       my $method = $opt->{id};
       my @args   = (! $method || $method eq 'AUTO') ? ( $tmp              )
@@ -76,22 +79,41 @@ sub _compile {
    $self->cache->id( $cache_id ); # if $cache_id;
    $self->[FILENAME] = $self->[TYPE] eq 'FILE' ? $tmpx : $self->cache->id;
 
+   my($shead, @sparam) = $opt->{_share} ? @{$opt->{_share}} : ();
+
+   LOG(
+      SHARED_VARS => "Adding shared variables ($shead) from a dynamic include"
+   ) if DEBUG && $shead;
+
    if ( not $ok ) {
       # we have a cache miss; parse and compile
       LOG( CACHE_MISS => $cache_id ) if DEBUG();
-      my $parsed = $self->_parse( $tmp, $opt->{map_keys}, $cache_id, $as_is  );
+
+      my $shared;
+      if ( $shead ) {
+         my $param = join ',', ('shift') x @sparam;
+         $shared = sprintf qq~my(%s) = (%s);~, $shead, $param;
+      }
+
+      local $self->[HEADER] = do {
+         my $old = $self->[HEADER] || '';
+         $shared . ';' . $old
+      } if $shared;
+
+      my %popt   = ( %{ $opt }, cache_id => $cache_id, as_is => $as_is );
+      my $parsed = $self->_parse( $tmp, \%popt );
       $CODE      = $self->cache->populate( $cache_id, $parsed, $opt->{chkmt} );
    }
 
-   my   @args;
-   push @args, $self if $self->[NEEDS_OBJECT];
+   my @args;
+   push @args, $self   if $self->[NEEDS_OBJECT]; # must be the first
+   push @args, @sparam if @sparam;
    push @args, @{ $self->[ADD_ARGS] } if $self->[ADD_ARGS];
    push @args, @{ $param };
    my $out = $CODE->( @args );
 
-   if ( $opt->{_filter} ) {
-      $self->_call_filters( \$out, split RE_FILTER_SPLIT, $opt->{_filter} );
-   }
+   $self->_call_filters( \$out, split RE_FILTER_SPLIT, $opt->{_filter} )
+      if $opt->{_filter};
 
    return $out;
 }
@@ -101,12 +123,12 @@ sub _call_filters {
    my $oref    = shift;
    my @filters = @_;
    my $fname   = $self->[FILENAME];
-   my $fbase   = 'Text::Template::Simple::Dummy';
 
    APPLY_FILTERS: foreach my $filter ( @filters ) {
-      my $fref = $fbase->can( "filter_" . $filter );
+      my $fref = DUMMY_CLASS->can( "filter_" . $filter );
       if ( ! $fref ) {
-         $$oref .= "\n[ filter warning ] Can not apply undefined filter $filter to $fname\n";
+         $$oref .= "\n[ filter warning ] Can not apply undefined filter"
+                .  " $filter to $fname\n";
          next;
       }
       $fref->( $self, $oref );
@@ -126,12 +148,6 @@ sub _wrap_compile {
 
    if( $error = $@ ) {
       my $error2;
-      if ( $self->[RESUME] ) {
-         $CODE =  sub {
-                     sprintf("[%s Fatal Error] %s", $self->_class_id, $error )
-                  };
-         $error2 = $@;
-      }
       $error .= $error2 if $error2;
    }
 
@@ -170,23 +186,23 @@ Private module.
 
 =head1 DESCRIPTION
 
-This document describes version C<0.70> of C<Text::Template::Simple::Base::Compiler>
-released on C<26 April 2009>.
+This document describes version C<0.80> of C<Text::Template::Simple::Base::Compiler>
+released on C<8 August 2009>.
 
 Private module.
 
 =head1 AUTHOR
 
-Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
+Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004 - 2009 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.8.8 or, 
+it under the same terms as Perl itself, either Perl version 5.10.0 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
