@@ -2,131 +2,31 @@ package Text::Template::Simple::Base::Parser;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.70';
+$VERSION = '0.80';
 
-use Text::Template::Simple::Util qw(:all);
+use Text::Template::Simple::Util      qw(:all);
 use Text::Template::Simple::Constants qw(:all);
 
-# internal code templates
-my %INTERNAL = (
-   # we need string eval in this template to catch syntax errors
-   sub_include => q~
-      <%OBJECT%>->_compile(
-         do {
-            local $@;
-            my $file = eval '<%INCLUDE%>';
-            my $rv;
-            if ( my $e = $@ ) {
-               chomp $e;
-               $file ||= '<%INCLUDE%>';
-               my $m = "The parameter ($file) is not a file. "
-                     . "Error from sub-include ($file): $e";
-               $rv = [ ERROR => '<%ERROR_TITLE%> ' . $m ]
-            }
-            else {
-               $rv = $file;
-            }
-            $rv;
-         },
-         <%PARAMS%>,
-         {
-            _sub_inc => '<%TYPE%>',
-            _filter  => '<%FILTER%>',
-         }
-      )
-   ~,
-   no_monolith => q*
-      <%OBJECT%>->compile(
-         q~<%FILE%>~,
-         undef,
-         {
-            chkmt    => 1,
-            _sub_inc => q~<%TYPE%>~,
-         }
-      );
-   *,
-
-   # see _parse()
-   map_keys_check => q(
-      <%BUF%> .= exists <%HASH%>->{"<%KEY%>"}
-               ? (
-                  defined <%HASH%>->{"<%KEY%>"}
-                  ? <%HASH%>->{"<%KEY%>"}
-                  : "[ERROR] Key not defined: <%KEY%>"
-                  )
-               : "[ERROR] Invalid key: <%KEY%>"
-               ;
-   ),
-
-   map_keys_init => q(
-      <%BUF%> .= <%HASH%>->{"<%KEY%>"} || '';
-   ),
-   map_keys_default => q(
-      <%BUF%> .= <%HASH%>->{"<%KEY%>"};
-   ),
-
-   add_sigwarn => q(
-      my <%BUF%>;
-      local $SIG{__WARN__} = sub {
-         push @{ <%BUF%> }, $_[0];
-      };
-   ),
-   dump_sigwarn => q(
-      join("\n",
-            map {
-               s{ \A \s+    }{}xms;
-               s{    \s+ \z }{}xms;
-               "[warning] $_\n"
-            } @{ <%BUF%> }
-         );
-   ),
-
-   compile_error => <<'TEMPLATE_CONSTANT',
-Error compiling code fragment (cache id: <%CID%>):
-
-<%ERROR%>
--------------------------------
-PARSED CODE (VERBATIM):
--------------------------------
-
-<%PARSED%>
-
--------------------------------
-PARSED CODE    (tidied):
--------------------------------
-
-<%TIDIED%>
-TEMPLATE_CONSTANT
-
-   fragment => <<'TEMPLATE_CONSTANT',
-
-# BEGIN TIDIED FRAGMENT
-
-<%FRAGMENT%>
-
-# END TIDIED FRAGMENT
-TEMPLATE_CONSTANT
-
-);
+my %INTERNAL = __PACKAGE__->_set_internal_templates;
 
 sub _internal {
    my $self = shift;
    my $id   = shift            || fatal('tts.base.parser._internal.id');
    my $rv   = $INTERNAL{ $id } || fatal('tts.base.parser._internal.id');
+   LOG( INTERNAL => "TEMPLATE: $id" ) if DEBUG;
    return $rv;
 }
 
 sub _parse {
-   my $self     = shift;
-   my $raw      = shift;
-   my $map_keys = shift; # code sections are hash keys
-   my $cache_id = shift;
-   my $as_is    = shift; # i.e.: do not parse -> static include
+   my($self, $raw, $opt) = @_;
+
+   # $opt->
+   #      map_keys: code sections are hash keys
+   #      as_is   : i.e.: do not parse -> static include
+
    #$self->[NEEDS_OBJECT] = 0; # reset
 
-   my $resume   = $self->[RESUME] || '';
-   my $ds       = $self->[DELIMITERS][DELIM_START];
-   my $de       = $self->[DELIMITERS][DELIM_END  ];
+   my($ds, $de) = @{ $self->[DELIMITERS] };
    my $faker    = $self->[INSIDE_INCLUDE] ? $self->_output_buffer_var
                                           : $self->[FAKER]
                                           ;
@@ -137,11 +37,11 @@ sub _parse {
    my $code     = '';
    my $inside   = 0;
 
-   my($mko, $mkc) = $self->_parse_mapkeys( $map_keys, $faker, $buf_hash );
+   my($mko, $mkc) = $self->_parse_mapkeys( $opt->{map_keys}, $faker, $buf_hash );
 
    LOG( RAW => $raw ) if ( DEBUG() > 3 );
 
-   my $uthandler = $self->[USER_THANDLER];
+   my $uth = $self->[USER_THANDLER];
 
    my $h = {
       raw     => sub { ";$faker .= q~$_[0]~;" },
@@ -150,14 +50,13 @@ sub _parse {
    };
 
    # little hack to convert delims into escaped delims for static inclusion
-   $raw =~ s{\Q$ds}{$ds!}xmsg if $as_is;
+   $raw =~ s{\Q$ds}{$ds!}xmsg if $opt->{as_is};
 
    # fetch and walk the tree
-   PARSER: foreach my $token ( @{ $toke->tokenize( $raw, $map_keys ) } ) {
+   PARSER: foreach my $token ( @{ $toke->tokenize( $raw, $opt->{map_keys} ) } ) {
       my($str, $id, $chomp, undef) = @{ $token };
       LOG( TOKEN => $toke->_visualize_tid($id) . " => $str" ) if DEBUG() > 1;
-      next PARSER if T_DISCARD == $id;
-      next PARSER if T_COMMENT == $id;
+      next PARSER if T_DISCARD == $id || T_COMMENT == $id;
 
       if ( T_DELIMSTART == $id ) { $inside++; next PARSER; }
       if ( T_DELIMEND   == $id ) { $inside--; next PARSER; }
@@ -167,18 +66,15 @@ sub _parse {
       }
 
       elsif ( T_CODE == $id ) {
-         $code .= $h->{code}->($resume ? $self->_resume($str, 0, 1) : $str);
+         $code .= $h->{code}->($str);
       }
 
       elsif ( T_CAPTURE == $id ) {
-         $code .= $faker;
-         $code .= $resume ? $self->_resume($str, RESUME_NOSTART)
-                :           " .= sub { $str }->();";
+         $code .= $h->{capture}->( $str );
       }
 
       elsif ( T_DYNAMIC == $id || T_STATIC == $id ) {
-         $self->[NEEDS_OBJECT]++;
-         $code .= $h->{capture}->( $self->_include($id, $str) );
+         $code .= $h->{capture}->( $self->_needs_object->_include($id, $str, $opt) );
       }
 
       elsif ( T_MAPKEY == $id ) {
@@ -186,33 +82,15 @@ sub _parse {
       }
 
       elsif ( T_COMMAND == $id ) {
-         my($head, $raw_block) = split /;/, $str, 2;
-         my @buf = split RE_PIPE_SPLIT, '|' . trim($head);
-         shift(@buf);
-         my %com = map { trim $_ } @buf;
-
-         if ( $com{FILTER} ) {
-            # embed into the template & NEEDS_OBJECT++ ???
-            local $self->[FILENAME] = '<ANON BLOCK>';
-            $self->_call_filters(
-               \$raw_block,
-               split RE_FILTER_SPLIT, $com{FILTER}
-            );
-         }
-
-         $code .= $h->{raw}->($raw_block);
+         $code .= $h->{raw}->( $self->_parse_command( $str ) );
       }
 
       else {
-         if ( $uthandler ) {
-            LOG( USER_THANDLER => "$id") if DEBUG();
-            $code .= $uthandler->( $self, $id ,$str, $h );
-         }
-         else {
-            LOG( UNKNOWN_TOKEN => "Adding unknown token as RAW: $id($str)")
-               if DEBUG();
-            $code .= $h->{raw}->($str);
-         }
+         LOG(
+            $uth  ? (USER_THANDLER => "$id")
+                  : (UNKNOWN_TOKEN => "Adding unknown token as RAW: $id($str)")
+         ) if DEBUG;
+         $code .= $uth ? $uth->( $self, $id ,$str, $h ) : $h->{raw}->( $str );
       }
 
    }
@@ -226,7 +104,24 @@ sub _parse {
          $self->[FILENAME]
    ) if $inside;
 
-   return $self->_wrapper( $code, $cache_id, $faker, $map_keys, $h );
+   return $self->_wrapper( $code, $opt->{cache_id}, $faker, $opt->{map_keys}, $h );
+}
+
+sub _parse_command {
+   my $self = shift;
+   my $str  = shift;
+   my($head, $raw_block) = split /;/, $str, 2;
+   my @buf  = split RE_PIPE_SPLIT, '|' . trim($head);
+   shift(@buf);
+   my %com  = map { trim $_ } @buf;
+
+   if ( $com{FILTER} ) {
+      # embed into the template & NEEDS_OBJECT++ ???
+      local $self->[FILENAME] = '<ANON BLOCK>';
+      $self->_call_filters( \$raw_block, split RE_FILTER_SPLIT, $com{FILTER} );
+   }
+
+   return $raw_block;
 }
 
 sub _chomp {
@@ -304,7 +199,7 @@ sub _wrapper {
    ) if DEBUG > 1;
    #LOG( OUTPUT => $wrapper );
    # reset
-   $self->[DEEP_RECURSION] = 0 if $self->[DEEP_RECURSION];
+   $self->[DEEP_RECURSION] = 0; # reset
    return $wrapper;
 }
 
@@ -379,41 +274,116 @@ sub _add_stack {
    return "$channel stack( { type => '$type', name => '$cs_name' } );";
 }
 
-# TODO: unstable. consider removing this thing (also the constants)
-sub _resume {
-   my $self    = shift;
-   my $token   = shift           || return;
-   my $nostart = shift           || 0;
-   my $is_code = shift           || 0;
-   my $resume  = $self->[RESUME] || '';
-   my $start   = $nostart ? '' : $self->[FAKER];
-   my $void    = $nostart ? 0  : 1; # not a self-printing block
-
-   if ( $token && $resume && $token !~ RESUME_MY ) {
-      if (
-            $token !~ RESUME_CURLIES &&
-            $token !~ RESUME_ELSIF   &&
-            $token !~ RESUME_ELSE    &&
-            $token !~ RESUME_LOOP
-      ) {
-         LOG( RESUME_OK => $token ) if DEBUG() > 2;
-         my $rvar        = $self->_output_buffer_var('array');
-         my $resume_code = RESUME_TEMPLATE;
-         foreach my $replace (
-            [ RVAR  => $rvar             ],
-            [ TOKEN => $token            ],
-            [ PID   => $self->_class_id  ],
-            [ VOID  => $void             ],
-         ) {
-            $resume_code =~ s{ <% $replace->[0] %> }{$replace->[1]}xmsg;
+sub _set_internal_templates {
+   # we need string eval in this template to catch syntax errors
+   sub_include => q~
+      <%OBJECT%>->_compile(
+         do {
+            local $@;
+            my $file = eval '<%INCLUDE%>';
+            my $rv;
+            if ( my $e = $@ ) {
+               chomp $e;
+               $file ||= '<%INCLUDE%>';
+               my $m = "The parameter ($file) is not a file. "
+                     . "Error from sub-include ($file): $e";
+               $rv = [ ERROR => '<%ERROR_TITLE%> ' . $m ]
+            }
+            else {
+               $rv = $file;
+            }
+            $rv;
+         },
+         <%PARAMS%>,
+         {
+            _sub_inc => '<%TYPE%>',
+            _filter  => '<%FILTER%>',
+            _share   => [<%SHARE%>],
          }
-         return $start . $resume_code;
-      }
-   }
+      )
+   ~,
 
-   LOG( RESUME_NOT => $token ) if DEBUG() > 2;
+   no_monolith => q*
+      <%OBJECT%>->compile(
+         q~<%FILE%>~,
+         undef,
+         {
+            chkmt    => 1,
+            _sub_inc => q~<%TYPE%>~,
+         }
+      );
+   *,
 
-   return $is_code ? $token : "$start .= $token;"
+   # see _parse()
+   map_keys_check => q(
+      <%BUF%> .= exists <%HASH%>->{"<%KEY%>"}
+               ? (
+                  defined <%HASH%>->{"<%KEY%>"}
+                  ? <%HASH%>->{"<%KEY%>"}
+                  : "[ERROR] Key not defined: <%KEY%>"
+                  )
+               : "[ERROR] Invalid key: <%KEY%>"
+               ;
+   ),
+
+   map_keys_init => q(
+      <%BUF%> .= <%HASH%>->{"<%KEY%>"} || '';
+   ),
+
+   map_keys_default => q(
+      <%BUF%> .= <%HASH%>->{"<%KEY%>"};
+   ),
+
+   add_sigwarn => q(
+      my <%BUF%>;
+      local $SIG{__WARN__} = sub {
+         push @{ <%BUF%> }, $_[0];
+      };
+   ),
+
+   dump_sigwarn => q(
+      join("\n",
+            map {
+               s{ \A \s+    }{}xms;
+               s{    \s+ \z }{}xms;
+               "[warning] $_\n"
+            } @{ <%BUF%> }
+         );
+   ),
+
+   compile_error => <<'TEMPLATE_CONSTANT',
+Error compiling code fragment (cache id: <%CID%>):
+
+<%ERROR%>
+-------------------------------
+PARSED CODE (VERBATIM):
+-------------------------------
+
+<%PARSED%>
+
+-------------------------------
+PARSED CODE    (tidied):
+-------------------------------
+
+<%TIDIED%>
+TEMPLATE_CONSTANT
+
+   fragment => <<'TEMPLATE_CONSTANT',
+
+# BEGIN TIDIED FRAGMENT
+
+<%FRAGMENT%>
+
+# END TIDIED FRAGMENT
+TEMPLATE_CONSTANT
+
+   disk_cache_comment => <<"TEMPLATE_CONSTANT",
+# !!!   W A R N I N G      W A R N I N G      W A R N I N G   !!!
+# This file was automatically generated by <%NAME%> on <%DATE%>.
+# This file is a compiled template cache.
+# Any changes you make here will be lost.
+#
+TEMPLATE_CONSTANT
 }
 
 1;
@@ -430,8 +400,8 @@ Private module.
 
 =head1 DESCRIPTION
 
-This document describes version C<0.70> of C<Text::Template::Simple::Base::Parser>
-released on C<26 April 2009>.
+This document describes version C<0.80> of C<Text::Template::Simple::Base::Parser>
+released on C<8 August 2009>.
 
 Private module.
 
@@ -461,16 +431,16 @@ happens.
 
 =head1 AUTHOR
 
-Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
+Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004 - 2009 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.8.8 or, 
+it under the same terms as Perl itself, either Perl version 5.10.0 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
