@@ -6,7 +6,7 @@ use Text::Template::Simple::Constants qw(:all);
 use Text::Template::Simple::Util qw( DEBUG LOG ishref fatal );
 use Carp qw( croak );
 
-$VERSION = '0.70';
+$VERSION = '0.80';
 
 my $CACHE = {}; # in-memory template cache
 
@@ -48,8 +48,8 @@ sub reset {
       my $file;
 
       while ( defined( $file = readdir CDIRH ) ) {
-         next if $file !~ m{ $ext \z}xmsi;
-         $file = File::Spec->catfile( $parent->[CACHE_DIR], $file );
+         next if $file !~ m{ ( .* $ext) \z}xmsi;
+         $file = File::Spec->catfile( $parent->[CACHE_DIR], $1 );
          LOG( UNLINK => $file ) if DEBUG();
          unlink $file;
       }
@@ -247,6 +247,19 @@ sub has {
    }
 }
 
+sub _is_meta_version_old {
+   my $self = shift;
+   my $v    = shift;
+   return 1 if ! $v; # no version? archaic then
+   my $pv = PARENT->VERSION;
+   foreach my $i ( $v, $pv ) {
+      $i  =~ tr/_//d; # underscore versions cause warnings
+      $i +=  0;       # force number
+   }
+   return 1 if $v < $pv;
+   return;
+}
+
 sub hit {
    # TODO: return $CODE, $META;
    my $self     = shift;
@@ -266,6 +279,12 @@ sub hit {
             %meta = $self->_get_meta( $1 );
             fatal('tts.cache.hit.meta', $@) if $@;
          }
+         if ( $self->_is_meta_version_old( $meta{VERSION} ) ) {
+            my $id = $parent->[FILENAME] || $cache_id;
+            warn "(This messeage will only appear once) $id was compiled with"
+                ." an old version of " . PARENT . ". Resetting cache.";
+            return;
+         }
          if ( my $mtime = $meta{CHKMT} ) {
             if ( $mtime != $chkmt ) {
                LOG( MTIME_DIFF => "\tOLD: $mtime\n\t\tNEW: $chkmt")
@@ -319,6 +338,7 @@ sub populate {
             CHKMT        => $chkmt,
             NEEDS_OBJECT => $parent->[NEEDS_OBJECT],
             FAKER_SELF   => $parent->[FAKER_SELF],
+            VERSION      => PARENT->VERSION,
          );
 
          my $cache = File::Spec->catfile( $cdir, $cache_id . CACHE_EXT);
@@ -326,12 +346,17 @@ sub populate {
          $fh->open($cache, '>') or fatal('tts.cache.populate.write', $cache, $!);
          flock $fh, Fcntl::LOCK_EX() if IS_FLOCK;
          $parent->io->layer($fh);
-         print $fh '#META:' . $self->_set_meta(\%meta) . "\n",
-                   sprintf( DISK_CACHE_COMMENT,
-                            PARENT->_class_id, scalar localtime time),
-                   $parsed; 
+         my $warn =  $parent->_mini_compiler(
+                        $parent->_internal('disk_cache_comment'),
+                        {
+                           NAME => PARENT->_class_id,
+                           DATE => scalar localtime time,
+                        }
+                     );
+         print $fh '#META:' . $self->_set_meta(\%meta) . "\n", $warn, $parsed; 
          flock $fh, Fcntl::LOCK_UN() if IS_FLOCK;
          close $fh;
+         chmod(CACHE_FMODE, $cache) || fatal('tts.cache.populate.chmod');
 
          ($CODE, $error) = $parent->_wrap_compile($parsed);
          LOG( DISK_POPUL => $cache_id ) if DEBUG() > 2;
@@ -405,8 +430,8 @@ TODO
 
 =head1 DESCRIPTION
 
-This document describes version C<0.70> of C<Text::Template::Simple::Cache>
-released on C<26 April 2009>.
+This document describes version C<0.80> of C<Text::Template::Simple::Cache>
+released on C<8 August 2009>.
 
 Cache manager for C<Text::Template::Simple>.
 
@@ -502,16 +527,16 @@ TODO
 
 =head1 AUTHOR
 
-Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
+Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004 - 2009 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.8.8 or, 
+it under the same terms as Perl itself, either Perl version 5.10.0 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
