@@ -2,25 +2,28 @@ package Text::Template::Simple::Cache::ID;
 use strict;
 use vars qw($VERSION);
 use overload q{""} => 'get';
-use Text::Template::Simple::Constants qw( MAX_FL );
-use Text::Template::Simple::Util      qw( DIGEST fatal );
+use Text::Template::Simple::Constants qw( MAX_FL RE_INVALID_CID );
+use Text::Template::Simple::Util      qw( LOG DEBUG DIGEST fatal );
 
-$VERSION = '0.70';
-
-my $RE_INVALID = qr{[^A-Za-z_0-9]};
+$VERSION = '0.80';
 
 sub new {
-   bless do { \my $anon }, shift;
+   my $class = shift;
+   my $self  = bless do { \my $anon }, $class;
+   return $self;
 }
 
 sub get { my $self = shift; $$self }
 sub set { my $self = shift; $$self = shift if defined $_[0]; return; }
 
 sub generate { # cache id generator
-   my $self   = shift;
-   my $data   = shift or fatal('tts.cache.id.generate.data');
-   my $custom = shift;
-   my $regex  = shift;
+   my($self, $data, $custom, $regex) = @_;
+
+   if ( ! $data ) {
+      fatal('tts.cache.id.generate.data') if ! defined $data;
+      LOG( IDGEN => "Generating ID from empty data" ) if DEBUG;
+   }
+
    $self->set(
       $custom ? $self->_custom( $data, $regex )
               : $self->DIGEST->add( $data )->hexdigest
@@ -31,15 +34,20 @@ sub generate { # cache id generator
 sub _custom {
    my $self  = shift;
    my $data  = shift or fatal('tts.cache.id._custom.data');
-   my $regex = shift || $RE_INVALID;
+   my $regex = shift || RE_INVALID_CID;
       $data  =~ s{$regex}{_}xmsg; # remove bogus characters
-   my $len   = length( $data );
-   if ( $len > MAX_FL ) { # limit file name length
-      $data = substr $data, $len - MAX_FL, MAX_FL;
-   }
+   my $len   = length $data;
+   # limit file name length
+   $data = substr $data, $len - MAX_FL, MAX_FL if $len > MAX_FL;
    return $data;
 }
 
+sub DESTROY {
+   my $self = shift || return;
+   LOG( DESTROY => ref $self ) if DEBUG();
+   return;
+}
+
 1;
 
 __END__
@@ -54,8 +62,8 @@ TODO
 
 =head1 DESCRIPTION
 
-This document describes version C<0.70> of C<Text::Template::Simple::Cache::ID>
-released on C<26 April 2009>.
+This document describes version C<0.80> of C<Text::Template::Simple::Cache::ID>
+released on C<8 August 2009>.
 
 TODO
 
@@ -79,16 +87,16 @@ Set the cache ID.
 
 =head1 AUTHOR
 
-Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
+Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004 - 2009 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.8.8 or, 
+it under the same terms as Perl itself, either Perl version 5.10.0 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
