@@ -2,7 +2,7 @@ package Text::Template::Simple::Constants;
 use strict;
 use vars qw($VERSION $OID $DID @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
-$VERSION = '0.70';
+$VERSION = '0.80';
 
 # object fields
 BEGIN { $OID = -1 } # init object field id counter
@@ -32,7 +32,6 @@ use constant INSIDE_INCLUDE   => ++$OID;
 use constant NEEDS_OBJECT     => ++$OID;
 use constant CID              => ++$OID;
 use constant FILENAME         => ++$OID;
-use constant RESUME           => ++$OID;
 use constant IOLAYER          => ++$OID;
 use constant STACK            => ++$OID;
 use constant USER_THANDLER    => ++$OID;
@@ -41,6 +40,7 @@ use constant INCLUDE_PATHS    => ++$OID;
 use constant PRE_CHOMP        => ++$OID;
 use constant POST_CHOMP       => ++$OID;
 use constant VERBOSE_ERRORS   => ++$OID;
+use constant TAINT_MODE       => ++$OID;
 use constant MAXOBJFIELD      =>   $OID; # number of the last object field
 
 # token type ids
@@ -61,13 +61,12 @@ use constant T_MAXID          =>   $DID;
 
 # settings
 use constant MAX_RECURSION    => 50; # recursion limit for dynamic includes
-use constant PARENT           => 'Text::Template::Simple';
+use constant PARENT           => ( __PACKAGE__ =~ m{ (.+?) ::Constants }xms );
 use constant IS_WINDOWS       => $^O eq 'MSWin32' || $^O eq 'MSWin64';
 use constant DELIM_START      => 0; # field id
 use constant DELIM_END        => 1; # field id
 use constant RE_NONFILE       => qr{ [ \n \r < > \* \? ] }xmso;
 use constant RE_DUMP_ERROR    => qr{Can\'t locate object method "first" via package "B::SVOP"};
-use constant RESUME_NOSTART   => 1;                   # bool
 use constant COMPILER         => PARENT.'::Compiler'; # The compiler
 use constant COMPILER_SAFE    => COMPILER.'::Safe';   # Safe compiler
 use constant DUMMY_CLASS      => PARENT.'::Dummy';    # Dummy class
@@ -88,6 +87,11 @@ use constant COLLAPSE_LEFT    => 0x000010;
 use constant COLLAPSE_RIGHT   => 0x000020;
 use constant COLLAPSE_ALL     => 0x000040;
 
+use constant TAINT_CHECK_NORMAL  => 0x000000;
+use constant TAINT_CHECK_ALL     => 0x000002;
+use constant TAINT_CHECK_WINDOWS => 0x000004;
+use constant TAINT_CHECK_FH_READ => 0x000008;
+
 # first level directives
 use constant DIR_CAPTURE      => '=';
 use constant DIR_DYNAMIC      => '*';
@@ -112,6 +116,8 @@ use constant TOKEN_CHOMP_PREV =>  1; # sub-key for TOKEN_CHOMP
 use constant LAST_TOKEN       => -1;
 use constant PREVIOUS_TOKEN   => -2;
 
+use constant CACHE_FMODE      => 0600;
+
 # SHA seems to be more accurate, so we'll try them first.
 # Pure-Perl ones are slower, but they are fail-safes.
 # However, Digest::SHA::PurePerl does not work under $perl < 5.6.
@@ -126,70 +132,14 @@ use constant DIGEST_MODS => qw(
    Digest::Perl::MD5
 );
 
-use constant RE_PIPE_SPLIT   => qr/ \| (?:\s+)? (PARAM|FILTER) : /xms;
+use constant RE_PIPE_SPLIT   => qr/ \| (?:\s+)? (PARAM|FILTER|SHARE) : /xms;
 use constant RE_FILTER_SPLIT => qr/ \, (?:\s+)? /xms;
-
-use constant DISK_CACHE_COMMENT => <<"TEMPLATE_CONSTANT";
-# !!!   W A R N I N G      W A R N I N G      W A R N I N G   !!!
-# This file is automatically generated by %s on %s.
-# This file is a compiled template cache.
-# Any changes you make here will be lost.
-#
-TEMPLATE_CONSTANT
+use constant RE_INVALID_CID  => qr{[^A-Za-z_0-9]}xms;
 
 use constant DISK_CACHE_MARKER => q{# This file is automatically generated by }
                                .  PARENT
                                ;
 
-use constant RESUME_TEMPLATE => sub {
-   my $tmp = q~
-         .= sub {
-                  local $SIG{__DIE__};
-                  my <%RVAR%> = [];
-                  push @{ <%RVAR%> }, eval { <%TOKEN%> };
-                  return qq([<%PID%> Fatal Error] $@) if $@;
-                  return "" if(<%VOID%>);
-                  return <%RVAR%>->[0] if @{<%RVAR%>} == 1;
-                  return +(@{<%RVAR%>});
-         }->();
-   ~;
-   $tmp =~ s/\n//xmsg;
-   $tmp =~ s/\s{2,}/ /xmsg;
-   return $tmp;
-}->();
-
-use constant RESUME_MY => qr{
-      # exclude my() declarations.
-      (?:
-         (?:my|local) (?:\s+|) \(       # my($foo)
-         |
-         (?:my|local) (?:\s+|) [\$\@\%] # my $foo
-         |
-         (?:my|local)[\$\@\%]           # my$foo
-      )
-      |
-      (?:
-         (?: unless|if|while|until|for|foreach )
-         (?:\s+|)
-         \(
-      )
-}xms;
-
-use constant RESUME_CURLIES => qr{
-   \A (?:\s+|) (?:[\{\}]) (?:\s+|) \z
-}xms;
-
-use constant RESUME_ELSIF => qr{
-   \A (?:\s+|) (?:\}) (?:\s+|) (?:else|elsif)
-}xms;
-
-use constant RESUME_ELSE => qr{
-   \A (?:\s+|) \} (?:\s+|)
-   else (?:\s+|) (?:\{) (?:\s+|) \z
-}xms;
-
-use constant RESUME_LOOP => qr{ (?:next|last|continue|redo) }xms;
-
 BEGIN {
    @ISA         = qw( Exporter );
 
@@ -206,7 +156,6 @@ BEGIN {
                         PARENT
                      )],
       templates =>   [qw(
-                        DISK_CACHE_COMMENT
                         DISK_CACHE_MARKER
                      )],
       delims    =>   [qw(
@@ -241,7 +190,6 @@ BEGIN {
                         NEEDS_OBJECT
                         CID
                         FILENAME
-                        RESUME
                         IOLAYER
                         STACK
                         USER_THANDLER
@@ -250,17 +198,9 @@ BEGIN {
                         PRE_CHOMP
                         POST_CHOMP
                         VERBOSE_ERRORS
+                        TAINT_MODE
                         MAXOBJFIELD
                      )],
-      resume    =>   [qw(
-                        RESUME_NOSTART
-                        RESUME_MY
-                        RESUME_CURLIES
-                        RESUME_ELSIF
-                        RESUME_ELSE
-                        RESUME_LOOP
-                        RESUME_TEMPLATE
-                     )],
       chomp     =>   [qw(
                         CHOMP_NONE
                         COLLAPSE_NONE
@@ -306,6 +246,12 @@ BEGIN {
                         T_COMMAND
                         T_MAXID
                       )],
+      taint     =>   [qw(
+                        TAINT_CHECK_NORMAL
+                        TAINT_CHECK_ALL
+                        TAINT_CHECK_WINDOWS
+                        TAINT_CHECK_FH_READ
+                     )],
       etc       =>   [qw(
                         DIGEST_MODS
                         STAT_MTIME
@@ -313,8 +259,10 @@ BEGIN {
                         RE_PIPE_SPLIT
                         RE_FILTER_SPLIT
                         RE_NONFILE
+                        RE_INVALID_CID
                         STAT_SIZE
                         MAX_RECURSION
+                        CACHE_FMODE
                      )],
    );
 
@@ -340,23 +288,23 @@ TODO
 
 =head1 DESCRIPTION
 
-This document describes version C<0.70> of C<Text::Template::Simple::Constants>
-released on C<26 April 2009>.
+This document describes version C<0.80> of C<Text::Template::Simple::Constants>
+released on C<8 August 2009>.
 
 Constants for Text::Template::Simple.
 
 =head1 AUTHOR
 
-Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
+Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004 - 2009 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.8.8 or, 
+it under the same terms as Perl itself, either Perl version 5.10.0 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
