@@ -10,7 +10,7 @@ use vars qw($VERSION);
 use Text::Template::Simple::Caller;
 use Text::Template::Simple::Util qw();
 
-$VERSION = '0.70';
+$VERSION = '0.80';
 
 sub stack { # just a wrapper
    my $opt = shift || {};
@@ -34,8 +34,8 @@ TODO
 
 =head1 DESCRIPTION
 
-This document describes version C<0.70> of C<Text::Template::Simple::Dummy>
-released on C<26 April 2009>.
+This document describes version C<0.80> of C<Text::Template::Simple::Dummy>
+released on C<8 August 2009>.
 
 All templates are compiled into this class.
 
@@ -59,16 +59,16 @@ See L<Text::Template::Simple::Caller> for more information.
 
 =head1 AUTHOR
 
-Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
+Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004 - 2009 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.8.8 or, 
+it under the same terms as Perl itself, either Perl version 5.10.0 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
