@@ -4,7 +4,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use Text::Template::Simple::Constants qw( :info DIGEST_MODS );
 use Carp qw( croak );
 
-$VERSION = '0.70';
+$VERSION = '0.80';
 
 BEGIN {
    if ( IS_WINDOWS ) {
@@ -31,7 +31,7 @@ BEGIN {
    %EXPORT_TAGS = (
       macro => [qw( isaref      ishref iscref                  )],
       util  => [qw( binary_mode DIGEST trim rtrim ltrim escape )],
-      debug => [qw( fatal       DEBUG  LOG                     )],
+      debug => [qw( fatal       DEBUG  LOG  L                  )],
    );
    @EXPORT_OK        = map { @{ $EXPORT_TAGS{$_} } } keys %EXPORT_TAGS;
    $EXPORT_TAGS{all} = \@EXPORT_OK;
@@ -65,15 +65,17 @@ my $lang = {
       'tts.main.init.thandler'                   => 'user_thandler parameter must be a CODE reference',
       'tts.main.init.include'                    => 'include_paths parameter must be a ARRAY reference',
       'tts.util.escape'                          => 'Missing the character to escape',
-      'tts.util.fatal'                           => '%s is not defined as an error',
       'tts.tokenizer.new.ds'                     => 'Start delimiter is missing',
       'tts.tokenizer.new.de'                     => 'End delimiter is missing',
       'tts.tokenizer.tokenize.tmp'               => 'Template string is missing',
+      'tts.tokenizer._get_symbols.regex'         => 'Regex is missing',
       'tts.io.validate.type'                     => 'No type specified',
       'tts.io.validate.path'                     => 'No path specified',
       'tts.io.validate.file'                     => 'validate(file) is not yet implemented',
       'tts.io.layer.fh'                          => 'Filehandle is absent',
       'tts.io.slurp.open'                        => "Error opening '%s' for reading: %s",
+      'tts.io.slurp.taint'                       => "Can't untaint FH",
+      'tts.io.hls.invalid'                       => 'FH is either absent or invalid',
       'tts.caller.stack.hash'                    => 'Parameters to stack() must be a HASH',
       'tts.caller.stack.type'                    => 'Unknown caller stack type: %s',
       'tts.caller._text_table.module'            => "Caller stack type 'text_table' requires Text::Table: %s",
@@ -85,6 +87,7 @@ my $lang = {
       'tts.cache.hit.meta'                       => 'Can not get meta data: %s',
       'tts.cache.hit.cache'                      => 'Error loading from disk cache: %s',
       'tts.cache.populate.write'                 => 'Error writing disk-cache %s : %s',
+      'tts.cache.populate.chmod'                 => 'Can not change file mode',
       'tts.base.compiler._compile.notmp'         => 'No template specified',
       'tts.base.compiler._compile.param'         => 'params must be an arrayref!',
       'tts.base.compiler._compile.opt'           => 'opts must be a hashref!',
@@ -98,28 +101,39 @@ my $lang = {
       'tts.base.examine._examine_type.extra'     => 'Type array has unknown extra fields',
       'tts.base.examine._examine_type.unknown'   => 'Unknown first argument of %s type to compile()',
       'tts.base.include._include.unknown'        => 'Unknown include type: %s',
+      'tts.base.include._interpolate.bogus_share' => 'Only SCALARs can be shared. You have tried to share a variable '
+                                                    .'type of %s named "%s". Consider converting it to a SCALAR or try '
+                                                    .'the monolith option to enable automatic variable sharing. '
+                                                    .'But please read the fine manual first',
       'tts.base.parser._internal.id'             => '_internal(): id is missing',
       'tts.base.parser._internal.rv'             => '_internal(): id is invalid',
       'tts.base.parser._parse.unbalanced'        => '%d unbalanced %s delimiter(s) in template %s',
       'tts.cache.id.generate.data'               => "Can't generate id without data!",
       'tts.cache.id._custom.data'                => "Can't generate id without data!",
    },
+   warning => {
+      'tts.base.include.dynamic.recursion'       => qq{%s Deep recursion (>=%d) detected in the included file: %s},
+   }
 };
 
-my $DEBUG = 0;  # Disabled by default
-my $DIGEST;     # Will hold digester class name.
+my $DEBUG = 0; # Disabled by default
+my $DIGEST;    # Will hold digester class name.
 
 sub isaref { $_[0] && ref($_[0]) && ref($_[0]) eq 'ARRAY' };
 sub ishref { $_[0] && ref($_[0]) && ref($_[0]) eq 'HASH'  };
 sub iscref { $_[0] && ref($_[0]) && ref($_[0]) eq 'CODE'  };
 
-sub fatal  {
-   my $ID  = shift;
-   my $str = $lang->{error}{$ID}
-             || croak sprintf( $lang->{error}{'tts.util.fatal'}, $ID );
-   croak @_ ? sprintf($str, @_) : $str;
+sub L {
+   my $type  = shift || croak "Type parameter to L() is missing";
+   my $id    = shift || croak "ID parameter ro L() is missing";
+   my @param = @_;
+   my $root  = $lang->{ $type } || croak "$type is not a valid L() type";
+   my $value = $root->{ $id }   || croak "$id is not a valid L() ID";
+   return @param ? sprintf($value, @param) : $value;
 }
 
+sub fatal { croak L( error => @_ ) }
+
 sub escape {
    my $c = shift || fatal('tts.util.escape');
    my $s = shift;
@@ -188,7 +202,7 @@ sub DIGEST {
       fatal( 'tts.util.digest' => join(', ', @report), $last, $@ );
    }
 
-   LOG( DIGESTER => $DIGEST ) if DEBUG();
+   LOG( DIGESTER => $DIGEST . ' v' . $DIGEST->VERSION ) if DEBUG();
    return $DIGEST->new;
 }
 
@@ -204,11 +218,12 @@ sub LOG {
 }
 
 sub _is_parent_object {
-   return 0 if not defined $_[0];
-   return 1 if         ref $_[0];
-   return 1 if             $_[0] eq __PACKAGE__;
-   return 1 if             $_[0] eq PARENT;
-   return 0;
+   return ! defined $_[0]       ? 0
+         : ref $_[0]            ? 1
+         : $_[0] eq __PACKAGE__ ? 1
+         : $_[0] eq PARENT      ? 1
+         :                        0
+         ;
 }
 
 1;
@@ -225,8 +240,8 @@ TODO
 
 =head1 DESCRIPTION
 
-This document describes version C<0.70> of C<Text::Template::Simple::Util>
-released on C<26 April 2009>.
+This document describes version C<0.80> of C<Text::Template::Simple::Util>
+released on C<8 August 2009>.
 
 Contains utility functions for Text::Template::Simple.
 
@@ -242,7 +257,11 @@ Returns the digester object.
 
 =head2 binary_mode FH, LAYER
 
-Sets the I/O layer of FH in moern perls, only sets binmode on FH otherwise.
+Sets the I/O layer of FH in modern perls, only sets binmode on FH otherwise.
+
+=head2 L TYPE, ID [, PARAMS]
+
+Internal method.
 
 =head2 fatal ID [, PARAMS]
 
@@ -291,16 +310,16 @@ the C<MYLOG> sub.
 
 =head1 AUTHOR
 
-Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
+Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004 - 2009 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.8.8 or, 
+it under the same terms as Perl itself, either Perl version 5.10.0 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
